require 'spec_helper'

describe Endorser::Response do

  describe '#initialize' do
    let(:raw) { double('RestClient::RawResponse') }
    let(:request) { double('RestClient::Request') }

    subject { Endorser::Response.new(raw, request) }

    its(:raw) { should eq(raw) }
    its(:request) { should eq(request) }
  end

  describe '#headers' do
    let(:raw_headers) { double('Hash') }
    let(:raw) { double('RestClient::RawResponse', raw_headers: raw_headers) }
    let(:response) { Endorser::Response.new(raw, nil) }

    subject { response.headers }
    it { should eq(raw_headers) }
  end

  describe '#cookies' do
    let(:cookies) { double('Hash') }
    let(:raw) { double('RestClient::RawResponse', cookies: cookies) }
    let(:response) { Endorser::Response.new(raw, nil) }

    subject { response.cookies }
    it { should eq(cookies) }
  end

  describe '#code' do
    let(:raw) { double('RestClient::RawResponse', code: 200) }
    let(:response) { Endorser::Response.new(raw, nil) }

    subject { response.code }
    it { should eq(200) }
  end


  describe '#informational?' do
    let(:raw) { double('RestClient::RawResponse') }
    let(:response) { Endorser::Response.new(raw, nil) }

    context 'when the response code is 100' do
      before { response.stub(code: 100) }
      subject { response.informational? }
      it { should be_true }
    end
    context 'when the response code is 199' do
      before { response.stub(code: 199) }
      subject { response.informational? }
      it { should be_true }
    end
    context 'when the response code is 99' do
      before { response.stub(code: 99) }
      subject { response.informational? }
      it { should be_false }
    end
    context 'when the response code is 200' do
      before { response.stub(code: 200) }
      subject { response.informational? }
      it { should be_false }
    end
  end

  describe '#success?' do
    let(:raw) { double('RestClient::RawResponse') }
    let(:response) { Endorser::Response.new(raw, nil) }

    context 'when the response code is 200' do
      before { response.stub(code: 200) }
      subject { response.success? }
      it { should be_true }
    end
    context 'when the response code is 299' do
      before { response.stub(code: 299) }
      subject { response.success? }
      it { should be_true }
    end
    context 'when the response code is 199' do
      before { response.stub(code: 199) }
      subject { response.success? }
      it { should be_false }
    end
    context 'when the response code is 300' do
      before { response.stub(code: 300) }
      subject { response.success? }
      it { should be_false }
    end
  end

  describe '#redirection?' do
    let(:raw) { double('RestClient::RawResponse') }
    let(:response) { Endorser::Response.new(raw, nil) }

    context 'when the response code is 300' do
      before { response.stub(code: 300) }
      subject { response.redirection? }
      it { should be_true }
    end
    context 'when the response code is 399' do
      before { response.stub(code: 399) }
      subject { response.redirection? }
      it { should be_true }
    end
    context 'when the response code is 299' do
      before { response.stub(code: 299) }
      subject { response.redirection? }
      it { should be_false }
    end
    context 'when the response code is 400' do
      before { response.stub(code: 400) }
      subject { response.redirection? }
      it { should be_false }
    end
  end

  describe '#client_error?' do
    let(:raw) { double('RestClient::RawResponse') }
    let(:response) { Endorser::Response.new(raw, nil) }

    context 'when the response code is 400' do
      before { response.stub(code: 400) }
      subject { response.client_error? }
      it { should be_true }
    end
    context 'when the response code is 499' do
      before { response.stub(code: 499) }
      subject { response.client_error? }
      it { should be_true }
    end
    context 'when the response code is 399' do
      before { response.stub(code: 399) }
      subject { response.client_error? }
      it { should be_false }
    end
    context 'when the response code is 500' do
      before { response.stub(code: 500) }
      subject { response.client_error? }
      it { should be_false }
    end
  end

  describe '#server_error?' do
    let(:raw) { double('RestClient::RawResponse') }
    let(:response) { Endorser::Response.new(raw, nil) }

    context 'when the response code is 500' do
      before { response.stub(code: 500) }
      subject { response.server_error? }
      it { should be_true }
    end
    context 'when the response code is 599' do
      before { response.stub(code: 599) }
      subject { response.server_error? }
      it { should be_true }
    end
    context 'when the response code is 499' do
      before { response.stub(code: 499) }
      subject { response.server_error? }
      it { should be_false }
    end
    context 'when the response code is 600' do
      before { response.stub(code: 600) }
      subject { response.server_error? }
      it { should be_false }
    end
  end

end
